﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;


public class GameController : MonoBehaviour {

	public AudioClip HitGroundClip;
	public AudioClip PlayerHitClip;
	public AudioClip EatClip;

	public Texture FullHeartTexture;
	public Texture EmptyHeartTexture;

	public Texture MenuTitleTexture;
	public Texture LevelTexture;
	public Texture SummaryTexture;
	public Texture TutorialTexture;

	public Font GUIFont;
	public int GUIFontSize;
	private bool guiInitialized=false;

	//public LevelData CurrentLevel { get; set; }
	public bool IsGameShuttingDown=false;

	public Transform Player;
	private PlayerStatus pstat;

	private enum GameState
	{
		MenuTitleScreen,
		LevelScreen,
		Running,
		SummaryScreen,
		TutorialScreen
	}
	private GameState state;
	private float lastSwitchTime;

	public System.Random rand;
	private LevelGenerator levelGenerator;
	private LevelInstantiator levelInstantiator;

	public int LevelNum=0;
	public bool Victory=false;
	
	void Awake() {
		DontDestroyOnLoad(transform.gameObject);
	}
	
	void Start () 
	{
		rand = new System.Random();

		GameObject pgo = GameObject.Find ("Player");
		Player = pgo.transform;
		pstat = pgo.GetComponent<PlayerStatus>();

		//CurrentLevel = new LevelData(0);
		//levelGenerator = new LevelGenerator(this);
		//levelInstantiator = new LevelInstantiator(this);
		iTween.CameraFadeAdd();
		EnterState (GameState.MenuTitleScreen);
	}
	
	void OnApplicationQuit() { 
		Debug.Log ("GameController.OnApplicationQuit");
		IsGameShuttingDown = true; 
	}
	
	void Update () 
	{
		switch(state)
		{
		case GameState.MenuTitleScreen:
			if(Input.GetKeyDown(KeyCode.Space))
			{
				//CurrentLevel = levelGenerator.GenerateLevel(1);
				LevelNum = 1;
				EnterState(GameState.LevelScreen);
			}
			break;

		case GameState.LevelScreen:
			if(Input.GetKeyDown(KeyCode.Space))
			{
				//levelInstantiator.InstantiateLevel(CurrentLevel);
				pstat.Health = pstat.MaxHealth;
				Application.LoadLevel(LevelNum);
				EnterState(GameState.Running);
			}
			break;

		case GameState.Running:
			if(pstat.Health <= 0)
			{
				Victory = false;
				EnterState (GameState.SummaryScreen);
			}
		
			GameObject[] gos = GameObject.FindGameObjectsWithTag("Target");
			if(gos.Length == 0 || Input.GetKeyDown(KeyCode.V))  // TODO remove this V skip key before shipping
			{
				Victory = true;
				EnterState (GameState.SummaryScreen);
			}
			break;

		case GameState.SummaryScreen:
			if(Input.GetKeyDown(KeyCode.Space))
			{
				if(Victory)
				{
					// TODO !@#$ CHECK FOR END OF GAME
					LevelNum++;
					if(LevelNum>7)
						LevelNum = 1;
					EnterState(GameState.LevelScreen);
				}
				else
					EnterState(GameState.LevelScreen);
			}
			break;
		}
	}

	void OnGUI()
	{
		if(!guiInitialized)
		{
			GUI.skin.label.font = GUIFont;
			GUI.skin.label.fontSize = GUIFontSize;
			GUI.skin.label.alignment = TextAnchor.UpperCenter;
			guiInitialized=true;
		}
				
		switch(state)
		{
		case GameState.MenuTitleScreen:
			//GUI.DrawTexture(new Rect(5,5, 600,600), MenuTitleTexture); //MenuTitleTexture.width, MenuTitleTexture.width),  MenuTitleTexture);
			GUIShowScreenCentered(0.3f, "FLYING PURPLE\n\na tale of vengeance\n\n\nby @pkenneydev\nludum dare 33\n\n\n\nInstructions:\nPress A/D or left/right keys\nto control purple");
			GUIShowScreenCentered(0.8f, "Press SPACE to Start");
			break;
		case GameState.LevelScreen:
			//GUI.DrawTexture(new Rect(5,5, 600,600),  LevelTexture);
			//GUI.TextArea(new Rect(20,20,80,80), "" + LevelNum);
			GUIShowScreenCentered(0.3f, ((LevelNum==1) ? "" : "Level " + LevelNum + "\n\n" )+ LevelFlavorText());
			GUIShowScreenCentered(0.8f, "Press SPACE to Start");			
			break;
		case GameState.Running:
			// ADD HUD CODE HERE
			ShowHeartHUD();
			break;
		case GameState.SummaryScreen:
			//GUI.DrawTexture(new Rect(5,5, 600,600),  SummaryTexture);
			if(Victory)
			{
				if(LevelNum == 7) {
					GUIShowScreenCentered(0.3f, "!!  CONGRATULATIONS  !!\n\n\nYou have deafeated the king\n\nand avenged your mother.\n\n\nIs that a gleam\n\nin her glass eye?");
					GUIShowScreenCentered(0.8f, "Press SPACE to Play Again");
				}
				else {
					GUIShowScreenCentered(0.3f, "Success!");
					GUIShowScreenCentered(0.8f, "Press SPACE to Forge Ahead");
				}
			}
			else {
				GUIShowScreenCentered(0.3f, "Failure!\n\n\nIt is tolerated\n\nin monsters");
				GUIShowScreenCentered(0.8f, "Press SPACE for Another Chance");
			}
			break;			
		}
	}

	public void GUIShowScreenCentered(float y, string msg, GUIStyle style = null)
	{
		if(style==null)
			GUI.Label(new Rect(0, y*Screen.height, Screen.width, (1-y)*Screen.height), msg); 
		else
			GUI.Label(new Rect(0, y*Screen.height, Screen.width, (1-y)*Screen.height), msg, style); 
	}

	public void GUIShowOffset(float x, float y, string msg, GUIStyle style = null)
	{
		int chars = msg.IndexOf("\n");
		if(chars == -1)
			chars = msg.Length;

		float wid = (float)chars * 0.026f;
		float size = Mathf.Min (Screen.width, Screen.height);

		// screen is from mid - size/2 to mid + size/2
		float sx = x*size + (Screen.width/2f - (size/2f));
		float sy = y*size + (Screen.height/2f - (size/2f));

		//Debug.Log ("Offset label at " + sx + ", " + sy + " wid " + wid);
		if(style == null)
			GUI.Label(new Rect(sx, sy, wid*size, Screen.height-sy), msg); 
		else
			GUI.Label(new Rect(sx, sy, wid*size, Screen.height-sy), msg, style); 
	}

	private void ShowHeartHUD()
	{
		float heartSize = 32f;
		for(int i = 0; i<pstat.MaxHealth; i++)
		{
			Texture tex = (pstat.Health>i) ? FullHeartTexture : EmptyHeartTexture;
			Rect rect = new Rect(1 + i*heartSize*1.2f, 3, heartSize, heartSize);
			GUI.DrawTexture(rect, tex);
			
			//GUI.DrawTexture(new Rect(7*Screen.width/8 + i*heartSize, 7*Screen.width/8, heartSize, heartSize), tex);
			//GUI.DrawTextureWithTexCoords(new Rect(7*Screen.width/8+heartHeight/10 + iconInset, nextBoxYOff+nextBoxHgt/10 + iconInset, mag*nextIconSpace, mag*nextIconSpace ), ntex, new Rect(0,0, 12f/(float)ntex.width,1));
		}
	}
	
	private void EnterState(GameState newState)
	{
		lastSwitchTime = Time.time;
		GameState oldState = state;
		state = newState;

		Debug.Log ("GameController Entering State " + state + " from " + oldState);

		switch(state)
		{
		case GameState.MenuTitleScreen:
			Player.gameObject.SetActive(false);
			break;
		case GameState.LevelScreen:
			Player.gameObject.SetActive(false);
			break;
		case GameState.Running:
			Player.gameObject.SetActive(true);
			Player.position = new Vector3(-58, 4, 0);
			Player.localEulerAngles = new Vector3(0,0,0);
			break;
		case GameState.SummaryScreen:
			Player.gameObject.SetActive(true);
			break;
		case GameState.TutorialScreen:
			Player.gameObject.SetActive(false);
			break;
			
		}

	}
	
	private string LevelFlavorText()
	{
		switch (LevelNum) 
		{
		case 1:
			return "VENGEANCE!  BLOOD!\n\n\n\nKing Wuncealot hath slewn thy mother\nand now he must pay.\n\n\n\nRaze his lands\nLeave nothing living\n\n\n\nAssault his castle\nand gnaw upon his bones.\n\n\n\n\n...But start with these sheep";
		break;
		case 2:
			return "Eat Everything\n\n\nEvery time";
		break;
		case 3:
			return "Man or Beast...\n\n\nWhat Difference,\nIn The End?";
		break;
		case 4:
			return "I wonder...\n\nWill these humans\ndefend themselves?";
		break;
		case 5:
			return "Not far now!";
			break;
		case 6:
			return "The Royale Guard...";
			break;
		case 7:
			return "King Wuncealot...\n\nThat monster!\n\n\n\nHe's mounted your mother's head\non his wall!\n\n\n\n\nHE MUST DIE!!";
			break;			
		}
		
		return "";
	}

	public void Shuffle<T>(IList<T> list)
	{
		for(int i=0; i < list.Count; i++)
			Swap(list, i, rand.Next(i, list.Count));
	}
	
	public void Swap<T>(IList<T> list, int i, int j)
	{
		var temp = list[i];
		list[i] = list[j];
		list[j] = temp;
	}

	public T DrawFromAtRandom<T>(IList<T> list)
	{
		if(list == null || list.Count == 0)
			return default(T);

		int idx = rand.Next(0, list.Count);
		return list[idx];
	}

	public T RemoveFromAtRandom<T>(IList<T> list)
	{
		T item = DrawFromAtRandom(list);
		//if(item != default(T))
			list.Remove(item);
		return item;

	}
}
