﻿using System;
using System.Linq;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;

[Serializable]
public struct StatusController
{
	public string StatusName;
	public List<AbstractControlExecutor> Executors;
}

[Serializable]
public struct StatusImpact
{
	public string StatusName;
	public float UntilTime;
}

public class StatusControlRouter : AbstractControlExecutor {

	public List<StatusController> StatusControllers;
	public List<AbstractControlExecutor> DefaultExecutors;
	
	public List<StatusImpact> StatusEffects = new List<StatusImpact>();
	
	public override bool ExecuteControls(BasicMoveController mctrl, Vector2 movePoint, GunSubController gsub, Transform GunTransform, Vector2 aimPoint, bool trig, bool secondaryTrig)
	{
		//Debug.Log("     "+gameObject.name + "StatusControlRouter ExecuteControls " + movePoint);
		
		bool done=false;
		
		// docs state the where/select does not alter order - prioritizes by order in editor
		var stats = StatusEffects.Where (s => s.UntilTime >= Time.fixedTime).Select(e => e.StatusName);	  // list of active status effects
		var ctrls = StatusControllers.Where (c => stats.Contains(c.StatusName));   // list of ctrls with active status
		var execs = ctrls.Select(c => c.Executors).Concat(new [] { DefaultExecutors } );  // get executor-lists, append defaults
		
		int i = 0;
		foreach(List<AbstractControlExecutor> ilist in execs)
		{
			//if(i<ctrls.Count())
			//	Debug.Log ("    " + gameObject.name + " running execs for status " + ctrls.ElementAt (i).StatusName);
			//else
				//Debug.Log ("    " + gameObject.name + " running default execs");
			
			foreach(AbstractControlExecutor ctrl in ilist)
			{
				if(ctrl.ExecuteControls(mctrl, movePoint, gsub, GunTransform, aimPoint, trig, secondaryTrig))
					done = true;
			}
			if(done) 
				return true;
			i++;
		}
		
		return done;
	}
	
	public void AddStatus(string name, float duration)
	{
		Debug.Log (gameObject.name + " Adding Status " + name + " for " + duration);
		
		StatusEffects.RemoveAll(s => s.StatusName == name);
		StatusEffects.Add (new StatusImpact { StatusName = name, UntilTime = Time.fixedTime + duration });
	}
}
