﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class EatTrigger : MonoBehaviour {
	
	public float ChewTime = 1f;
	public Transform HuntPoint;
	public JuiceDropper ChewJuiceDropper;
	
	private Rigidbody2D r2;
	private float chewUntil = 0;
	private List<Transform> eatList;
	
	private Animator anim;
	private PlayerStatus pstat;
	
	private bool hunting;
	private bool chewing;
	private bool stunned;
	private bool dead;
	private GameController gc;
	
	// Use this for initialization
	void Start () {
		GameObject go = GameObject.Find ("GameController");
		gc = go.GetComponent<GameController>();
		r2 = transform.parent.parent.GetComponent<Rigidbody2D>();
		anim = transform.parent.GetComponent<Animator>();
		eatList = new List<Transform>();
		pstat = transform.parent.parent.GetComponent<PlayerStatus>();
	}
	
	void FixedUpdate () {
		
		hunting = false;
		stunned = false;
		dead = false;
		
		if(pstat.Health<=0)
		{
			dead = true;
			chewing = false;
		}
		else if(pstat.IsStunned())
		{
			stunned = true;
			chewing = false;
			UpdateCarry(); // TODO implement actually dropping stuff when stunned?
		}		
		else if(chewUntil > Time.time)  // still chewing
		{
			UpdateCarry();
			chewing = true;
		}
		else
		{
			chewing = false;
			if(eatList.Count>0)
			{
				Swallow();
			}
			else
			{
				CheckHunting();
			}
		}
		
		anim.SetBool("chewing", chewing);
		anim.SetBool("hunting", hunting);	
		anim.SetBool("stunned", stunned);	
		anim.SetBool("dead", dead);	
	}
	
	void OnTriggerEnter2D(Collider2D col)
	{
		if(pstat.Health<=0)
			return;
			
		if(col.tag == "Target")
		{
			//Debug.Log("Ate the Target!  " + col.gameObject.name + " at vel " + r2.velocity.magnitude);
			Eat (col.transform);
		}
	}
	
	private void Eat(Transform t)
	{
		eatList.Add (t);
		chewUntil = Time.time + ChewTime;
		t.gameObject.layer = LayerMask.NameToLayer("Ignore");  // stuff in mouth shouldn't collide, causes probs
		ChewJuiceDropper.StartDropping();
		
		TargetMover tm = t.GetComponent<TargetMover>();
		if(tm!=null)
			tm.enabled = false;
			
		Archer ar = t.GetComponent<Archer>();
		if(ar!=null)
			ar.enabled = false;
		
		KingMover k = t.GetComponent<KingMover>();
		if(k!=null)
			k.enabled = false;
		
		iTween.Stab (gameObject, gc.EatClip, 0);
	}
	
	private void Swallow()
	{
		foreach(Transform t in eatList.ToArray())
		{
			eatList.Remove (t);
			GameObject.Destroy(t.gameObject);
		}
	}
	
	private void UpdateCarry()
	{
		foreach(Transform t in eatList)
		{
			t.position = transform.position;
		}
	}
	
	private void CheckHunting()
	{
		// TODO use Physics2D.OverlapArea to check for upcoming targets and open mouth for them!
		// assing hunting
		
		LayerMask mask = LayerMask.GetMask ("Target");
		Collider2D col = Physics2D.OverlapCircle(HuntPoint.position, 5f, mask);
		hunting = (col != null);
	}
}
