﻿using UnityEngine;
using System.Collections;

public class FlightController : MonoBehaviour {

	public float TurnRampTime=0.2f;
	
	public float Lift = 13f;
	public float Propel = 6f;
	public float TurnSpeed = 45f;
	public string DSTRING="";
	
	public Vector2 Controls;
	private float turningTime;
	
	private PlayerStatus pstat;

	// Use this for initialization
	void Start () {
		pstat = gameObject.GetComponent<PlayerStatus>();
		
	}
	
	void FixedUpdate () {
		GetComponent<Rigidbody2D>().fixedAngle = (pstat.Health>0);
	
		if(transform.position.y > 12f) // too high, override player control
		{
			float s = Mathf.Sin (Mathf.Deg2Rad*transform.localEulerAngles.z);		
			bool alreadydiving = s < -0.35f;
			
			if(!alreadydiving)
			{
				int dir = -1;
				if(transform.localScale.y == -1)
					dir = 1;
				
				float correction = TurnSpeed * 0.7f * dir * Time.fixedDeltaTime;
				transform.localEulerAngles	+= new Vector3(0,0,correction);	
			}
		}
		else if(!pstat.IsStunned())
		{
			if(Controls.x == 0)
				turningTime = 0;
			else
				turningTime += Time.fixedDeltaTime;
		
			float ramp = 0.2f + 0.8f*Mathf.Min (1,turningTime/TurnRampTime);
		
			float newRot = transform.localEulerAngles.z  + (TurnSpeed * -Controls.x * ramp * Time.fixedDeltaTime);
			transform.localEulerAngles = new Vector3(0,0,newRot);			
		}
				
		float z = transform.localEulerAngles.z;
		float newYscale = 1;		
		if(z>=90 && z <= 270) 
			newYscale = -1;
		
		transform.localScale = new Vector3(1,newYscale,1);
		
		if(pstat.Health>0)
		{
			ApplySlipForceForVelocity();
			ApplyCoastForce();
		}						
	}
	
	private void ApplySlipForceForVelocity()
	{
	    Rigidbody2D r2 = GetComponent<Rigidbody2D>();
		Vector2 vel = (Vector2)r2.velocity;
		float dot = Vector2.Dot((Vector2)transform.right, vel);
		Vector2 projection = dot * (Vector2)transform.right;  // portion of vel that is aligned with wing
	    Vector2 force = -(vel - projection);  // force of wing fighting vel

	    //float vconversion = 0.9f;
	    	    	    
//	    bool op = oppSign(transform.right.x, force.x);
//	    if(op && Mathf.Abs (vel.x)<0.5f) {
//			force = new Vector2(0,0.5f*force.y);  // this prevents "gliding backward" if we bank up
//			vconversion = 0.6f;
//	    }

		//if(force.y<0)
		//	force = new Vector2(force.x,0);
	    
		//DSTRING = "vel: "+vel+", fwd: "+(Vector2)transform.right+", dot="+dot+", proj="+projection+", force = "+force;
		
		r2.AddForce(0.275f * force * r2.mass/Time.fixedDeltaTime);
	}
	
//	private bool oppSign(float x, float y)
//	{
//		return (x*y)<0;
//	}


	private void ApplyCoastForce()
	{
		bool faceup = ( transform.position.y < (transform.position + transform.right).y );
	
		// TODO	
		Rigidbody2D r2 = GetComponent<Rigidbody2D>();
		Vector2 vel = (Vector2)r2.velocity;
		float dot = Vector2.Dot((Vector2)transform.right, vel);
		Vector2 projection = dot * (Vector2)transform.right;  // portion of vel that is aligned with wing
		
		if(projection.magnitude < 14)
		{
			float factor = Mathf.Abs (Mathf.Cos(Mathf.Deg2Rad*transform.localEulerAngles.z));
			if(faceup) {
				if(transform.position.y < -0.5f)
					factor = 1;    // keep full lift out of a crash (v near ground)
				else
					factor = 0.25f + 0.75f*factor; // keep some upward power for climbing
			}
			Vector2 force = new Vector2(350,0);
			r2.AddRelativeForce(factor * force);
			DSTRING = "factor: " + factor;
		}
	}
}
