﻿using UnityEngine;
using System.Collections;

public class TargetMover : MonoBehaviour {

	public float WalkSpeed=3f;
	public float SwitchDuration=2f;
	public float SwitchVariance=5f;
	public float FearScan=3f;
	public float AccelDampFactor = 0.1f;
	public float StandStillChance = 0.25f;
	public float XMin;
	public float XMax;
	public bool ChasePlayer=false;
	public float ChaseRange=10f;
	
	private int direction;
	private float nextSwitch;
	private float lastSwitch;
	
	private float afraidUntil=0;

	private GameController gc;
	private Rigidbody2D r2;
	private Vector3 startScale;
	
	public string REASON="";
	public string CHASEINFO="";
	
	void Start () {
		GameObject go = GameObject.Find ("GameController");
		gc = go.GetComponent<GameController>();
		r2 = transform.GetComponent<Rigidbody2D>();
		startScale = transform.localScale;
		
		SetRandomDirection();			
		SwitchDirection();
	}
	
	// Update is called once per frame
	void FixedUpdate () {
		if(OverrideForLimit())
		{
			//Debug.Log (gameObject.name + " switching to avoid limit");
			REASON = "limit";
			Walk();
			return;
		}
				
		if(SwitchInFear())
		{
			//Debug.Log (gameObject.name + " switching in fear");
			REASON = "fear";
			afraidUntil = Time.time + 2f;
			SwitchDirection();
			Walk();
			return;
		}
		
		if(Time.time > afraidUntil && SwitchedForChase())
		{
			//Debug.Log (gameObject.name + " switching to chase the player");
			REASON = "chase";
			Walk();
			return;
		}
		
		if(Time.time >= nextSwitch)
		{
			REASON = "time";
			SwitchDirection();
		}
		Walk();
		
		
	}
	
	private void Walk()
	{
		if(direction == 0)
			return;
			
		transform.localScale = new Vector3(direction * -startScale.x, startScale.y, startScale.z);
			
		Vector2 targetspd = new Vector2(WalkSpeed*direction,0);
		Vector2 delta = targetspd - (Vector2)r2.velocity;
		Vector2 force = AccelDampFactor * r2.mass * delta / Time.fixedDeltaTime;
		force = new Vector2(force.x, 0);
				
		r2.AddForce(force);
	}
	
	private bool SwitchedForChase()
	{
		if(!ChasePlayer || (Time.time - lastSwitch < 1f))
			return false;
		
		float diff = gc.Player.position.x - transform.position.x;
		CHASEINFO="df "+diff + " dr "+direction;
		if(Mathf.Abs (diff) > ChaseRange || Mathf.Abs (diff) < FearScan)
			return false;
			
		if(diff<0 && direction >=0)
		{
			direction = -1;
			nextSwitch = Time.time + SwitchDuration + (float)gc.rand.NextDouble()*SwitchVariance;
			lastSwitch = Time.time;
			return true;
		}
		if(diff>0 && direction <=0)
		{
			direction = 1;
			nextSwitch = Time.time + SwitchDuration + (float)gc.rand.NextDouble()*SwitchVariance;
			lastSwitch = Time.time;
			return true;
		}
		
		return false;
	}
	
	private bool SwitchInFear()
	{
		float xst  = (direction==0) ? transform.position.x - 0.5f*FearScan : transform.position.x;
		float xoff = (direction==0) ? transform.position.x + 0.5f*FearScan : transform.position.x + direction*FearScan;
				
		LayerMask mask = LayerMask.GetMask ("Player");
		Collider2D col = Physics2D.OverlapArea( new Vector2(xst,  transform.position.y+2),
												new Vector2(xoff, transform.position.y-2), mask);
		
		return (col != null);
	}
	
	private bool OverrideForLimit()
	{
		if(transform.position.x <= XMin)
		{
			direction = 1;
			nextSwitch = Time.time + SwitchDuration + (float)gc.rand.NextDouble()*SwitchVariance;
			lastSwitch = Time.time;
			return true;
		}
		if(transform.position.x >= XMax)
		{
			direction = -1;
			nextSwitch = Time.time + SwitchDuration + (float)gc.rand.NextDouble()*SwitchVariance;
			lastSwitch = Time.time;
			return true;
		}
		return false;
	}
	
	private void SwitchDirection()
	{	
		nextSwitch = Time.time + SwitchDuration + (float)gc.rand.NextDouble()*SwitchVariance;
		lastSwitch = Time.time;
		
		if(direction==0)
		{
			SetRandomDirection();
		}
		else
		{		
			bool still = ((float)gc.rand.NextDouble() < StandStillChance);
			if(still)
				direction = 0;
			else
				direction *= -1;
		}
	}
	
	private void SetRandomDirection()
	{
		direction = 1;
		if(gc.rand.Next(0,2) == 0)
			direction = -1;
	}
}
